#
# shinGETsu - P2P anonymous BBS
#
# (c) Fuktommy <fuktommy@users.sourceforge.net>
# Released under the GNU General Public License
# $Id: AttachCache.pm,v 1.3 2004/08/25 15:20:19 fuktommy Exp $
#

package Shingetsu::AttachCache;
use strict;
use Shingetsu::Config;
use Shingetsu::Extern;
use Shingetsu::Signature;
use Shingetsu::Util;

#
# configration
#
my $datadir  = $Shingetsu::Config::datadir;
my $cachedir = $Shingetsu::Config::cachedir;

#
# save attach as file
#
sub save($%) {
	my($cache, %rec) = @_;
	my $suffix = $rec{suffix};
	$suffix = "txt" unless ((defined $suffix) && ($suffix ne ""));
	$suffix =~ s/[^-_\w]//g;
	$suffix = "txt" unless ($suffix ne "");
	my $file = "$cache/attach/$rec{stamp}_$rec{id}.$suffix";
	unless (-s $file) {
		my $attach = Shingetsu::Extern::base64decode($rec{attach});
		open ATTACH, "> $file";
		print ATTACH $attach;
		close ATTACH;
	}
}

#
# add record
#
sub addRecord($$;%) {
	my ($file, $line, %rec) = @_;
	my $theCache = "$cachedir/$file";
	foreach ($theCache, "$theCache/record", "$theCache/attach") {
		mkdir $_, 0777 unless (-d $_);
	}
	if (keys %rec == 0) {
		%rec = Shingetsu::Util::rec($line);
	}
	Shingetsu::Util::lock();
	open BODY, ">> $theCache/body.cache";
	open HEAD, ">> $theCache/head.cache";
	if ($rec{attach}) {
		save($theCache, %rec);
	}
	open REC, "> $theCache/record/$rec{stamp}_$rec{id}.cache";
	print REC $line;
	close REC;
	$line =~ s|<>attach:[A-Za-z0-9+/=]+|<>attach:1|;
	$line =~ s|<>pubkey:([A-Za-z0-9+/=]+)|"<>pubkey:".Shingetsu::Extern::pubkey2trip($1)|e;
	if ($rec{pubkey} && Shingetsu::Signature::check(%rec)) {
		$line =~ s|<>sign:([A-Za-z0-9+/=]+)|<>sign:1|;
	} else {
		$line =~ s|<>sign:([A-Za-z0-9+/=]+)||;
	}

	print BODY $line, "\n";
	print HEAD "$rec{stamp}<>$rec{id}\n";
	close BODY;
	close HEAD;
	Shingetsu::Util::unlock();
}

#
# add new data cache
#
sub update($@) {
	my($file, @data) = @_;
	my $theCache = "$cachedir/$file";
	local $_;
	foreach ($theCache, "$theCache/record", "$theCache/attach") {
		mkdir $_, 0777 unless (-d $_);
	}
	if ((-s "$theCache/body.cache") && (-s "$theCache/head.cache")) {
		my @body = stat "$theCache/body.cache";
		my @head = stat "$theCache/head.cache";
		my @orig = stat "$datadir/$file.dat";
		if (($body[9] >= $orig[9]) && ($head[9] >= $orig[9])) {
			return;
		}
	}
	open BODY, "> $theCache/body.cache";
	open HEAD, "> $theCache/head.cache";
	foreach (@data) {
		chomp;
		my %rec = Shingetsu::Util::rec($_);
		next if (keys %rec <= 2);
		if ($rec{attach}) {
			save($theCache, %rec);
		}
		unless (-s "$theCache/record/$rec{stamp}_$rec{id}.cache") {
			open REC, "> $theCache/record/$rec{stamp}_$rec{id}.cache";
			print REC $_;
			close REC;
		}
		s|<>attach:[A-Za-z0-9+/=]+|<>attach:1|;
		s|<>pubkey:([A-Za-z0-9+/=]+)|"<>pubkey:".Shingetsu::Extern::pubkey2trip($1)|e;
		if ($rec{pubkey} && Shingetsu::Signature::check(%rec)) {
			s|<>sign:([A-Za-z0-9+/=]+)|<>sign:1|;
		} else {
			s|<>sign:([A-Za-z0-9+/=]+)||;
		}
		print BODY $_, "\n";
		print HEAD "$rec{stamp}<>$rec{id}\n";
	}
	close BODY;
	close HEAD;
}

#
# remove record in cache file
#
sub removeCacheRecord($$$) {
	my($file, $stamp, $id) = @_;
	open IN, $file or return 0;
	Shingetsu::Util::lock();
	my @data = <IN>;
	close IN;
	open OUT, "> $file" or return 0;;
	foreach (@data) {
		unless (/^$stamp<>$id/) {
			print OUT;
		}
	}
	close OUT;
	Shingetsu::Util::unlock();
	return 1;
}

#
# remove record
#
sub removeRecord($$$) {
	my($file, $stamp, $id) = @_;
	my $theCache = "$cachedir/$file";
	local $_;
	my $code = 1;

	$code &= removeCacheRecord("$theCache/body.cache", $stamp, $id);
	$code &= removeCacheRecord("$theCache/head.cache", $stamp, $id);

	foreach (glob "$theCache/*/${stamp}_${id}.*") {
		$code &= unlink $_;
	}
	return $code;
}

#
# remove file
#
sub removeFile($) {
	my($file) = @_;
	my $theCache = "$cachedir/$file";
	local $_;
	my $code = 1;

	foreach (glob("$theCache/*/*"), glob("$theCache/*.cache")) {
		$code &= unlink $_;
	}
	foreach (glob("$theCache/*/"), $theCache) {
		rmdir $_;
	}
	return $code;
}

#
# check original data and cache data
#
sub check() {
	local $_;
	my $d;
	foreach $d (glob "$cachedir/*") {
		my $dat = $d;
		$dat =~ s|$cachedir/||;
		unless (-s "$datadir/$dat.dat") {
			my $f;
			foreach $f ("$d/*") {
				unlink $f;
			}
			rmdir $d;
			next;
		}
		my %exist = ();
		open IN, "$d/head.cache";
		while (<IN>) {
			chomp;
			$exist{$_} = 1;
		}
		close IN;
		my $c;
		foreach $c (glob "$d/*") {
			if ($c =~ m|^$d/(\d+)_([0-9a-f]+)\.|) {
				my ($stamp, $id) = ($1, $2);
				unless ($exist{"$stamp<>$id"}) {
					unlink $c;
				}
			}
		}
	}
	warn "AttachCache::check() finished\n";
}

1;
